/*
 * Author: Benedikt Sauter <sauter@ixbat.de>
 * All rights reserved.
 *
 * Short descripton of file:
 *
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution.
 *   * Neither the name of the FH Augsburg nor the names of its 
 *     contributors may be used to endorse or promote products derived 
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef SPI_H
#define SPI_H

#include <avr/io.h>
#include <avr/pgmspace.h>


#include "io.h"


#define CS (B, 0)
#define SK (B, 1)
#define DI (B, 2)
#define DO (B, 3)






#define DIN0 (B, 4)
#define DIN1 (B, 5)

#define CMD_READ    0x80   
#define CMD_EWEN    0x30    
#define CMD_ERASE   0xc0  
#define CMD_WRITE   0x40      
#define CMD_ERAL    0x20     
#define CMD_WRAL    0x10      
#define CMD_EWDS    0x00    


#define PIN_S	PB0
#define PIN_C	PB1
#define PIN_D	PB2
#define PIN_Q	PB3	

void flash_93c46_parser(char *buf);

int flash_93c46_init(void);
int flash_93c46_deinit(void);

int flash_93c46_read(unsigned char address, unsigned int length, char * buf);
int flash_93c46_write(unsigned char address, unsigned int length, char* buf);

void flash_93c46_init_usb(void);
void flash_93c46_deinit_usb(void);

void flash_93c46_read_usb(unsigned char address, unsigned int length, char* buf);
void flash_93c46_write_usb(unsigned char address, unsigned int length, char* buf);


void start_of_ins(void);
void send_sb_opcode(uint8_t ins);
void send_data_byte(uint8_t da);
uint8_t end_of_erase_write(void);
void end_of_erase_write_disable(void);
void erase_write_enable(void);
void send_address(uint8_t da);
uint8_t write_word_step(uint8_t add,uint8_t wd);
uint8_t recv_data_word(void);
uint8_t erase_word(uint8_t add);
void erase_write_disable(void);



uint8_t flash_93c46_read_word(uint8_t add);
uint8_t flash_93c46_write_word(uint8_t add,uint8_t wd);











#endif /* SPI_H */
