/*
 * Copyright (c) 2006, Benedikt Sauter <sauter@empronix.com>
 * All rights reserved.
 *
 * Short descripton of file:
 *
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution.
 *   * Neither the name of the FH Augsburg nor the names of its 
 *     contributors may be used to endorse or promote products derived 
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <avr/io.h>
#include <stdint.h>
#include <avr/interrupt.h>
#include <inttypes.h>

#include "common.h"
#include "protocol.h"
#include "io.h"
#include "93c46.h"



void common_latency_timer_init(void)
{
   // us = 256 - x/2
   octopus.latency_timer = 206; // 255 = 2us, 254 = 4us ...
   
   #ifdef OCTOPUS_CAN
   TIMSK0 = (1<<TOIE0);  /* enable TCNT0 overflow */
   TCNT0 = octopus.latency_timer;    /* TCNT0 Timer auf 50ms einstellen */
   TCCR0A = 3;          /* count with cpu clock/32 */ 
   #else 
   TIMSK = (1<<TOIE0);  /* enable TCNT0 overflow */
   TCNT0 = octopus.latency_timer;    /* TCNT0 Timer auf 50ms einstellen */
   TCCR0 = 3;          /* count with cpu clock/32 */ 
   #endif
}

void common_scheduler(void){}

void common_scheduler_add(int id,void(*userFunc)(void)){}


void common_parser(char *buf)
{
   switch(buf[0])
   {
      case CMD_GET_HW_ID:
         common_hwdesc();
      break;
      default:
         ;
   }
}

void common_init(void)
{
   int i;
   for(i=1;i<=19;i++)
      io_init(i);
   for(i=26;i<=44;i++)
      io_init(i);
}


void common_hwdesc(void)
{
   answer[0] = CMD_GET_HW_ID;
   answer[1] = RSP_OK;
   answer[2] = 11;
   answer[3] = 'o'; answer[4] = 'c';answer[5] = 't';answer[6] = 'o';
   #ifdef OCTOPUS_CAN  
   answer[7] = 'c'; answer[8] = 'a';answer[9] = 'n';answer[10] = '_';
   #else
   answer[7] = 'p'; answer[8] = 'u';answer[9] = 's';answer[10] = '_';
   #endif  
   answer[11] = '0'; answer[12] = '1';
   answer[13] = 0x00;
   CommandAnswer(14);
}


