#include <inttypes.h>
#include <avr/io.h>

#include "uart.h"

void UARTInit2(int baud,char data, char parity, char stop)
{
    int ubbr=(16000000/16/baud)-1; // (FOSC/(16*baud))-1 
    char upm1=0,upm0=0,usbs=0,ucsz1=0,ucsz0=0;

    switch(data){
        case 5: ucsz1=0; ucsz0=0; break;
        case 6: ucsz1=0; ucsz0=1; break;
        case 7: ucsz1=1; ucsz0=0; break;
        case 8: ucsz1=1; ucsz0=1; break;
        default: ucsz1=1; ucsz0=1; break;
    }

    switch(parity)
    {
        case 'N': upm1=0; upm0=0; break;
        case 'E': upm1=1; upm0=0; break;
        case 'O': upm1=1; upm0=1; break;
        default: upm1=0; upm0=0; break;
    }

    switch(stop)
    {
        case 1: usbs=0; break;
        case 2: usbs=2; break;
        default: usbs=0; break;
    }

    UBRR0H=ubbr>>8;
    UBRR0L=ubbr;
    UCSR0B = (1 << RXEN0) | (1 << TXEN0);               // enable transmitter receiver
    UCSR0C= (upm1 << UPM01)|(upm0 <<UPM00)|(usbs<<USBS0)|(ucsz1 << UCSZ01)|(ucsz0 << UCSZ00);

}

/**
 * Init function for USART0
 **/
 #define BAUDRATE 38400
 #define PTAKT 40000000
 #define UBFORMEL ((PTAKT/4/BAUDRATE)-1)
 
void UARTInit(void)
{
  /* Baudrate = 9600 Baud, F_OSC = 4Mhz, U2X=0 (normal speed) => value for UBBR = 25 decimal , 0b0000 00011001
   * Formular for calculation on page 175, Examples of the most common UBBR Settings on pages 195ff
   * */
   UBRR0H = (unsigned char) 0; // 4 most significant bits of 25 = 0b0000 =0
  //UBRR0L = (unsigned char) 103; // 8 least significant bits = 0b11001 = 25
   UBRR0L = (unsigned char) 25; // 8 least significant bits = 0b11001 = 25 // bei 4 MHz (lfuse 0xe4) 13 = 38400 baud
   
  
  //UBRR0H = (unsigned char) (UBFORMEL >> 8);
  //UBRR0L = (unsigned char) (UBFORMEL);

  /* Enable receiver and transmitter and Receive interupt*/
  //UCSR0B = (1 << RXEN0) | (1 << TXEN0) | (1 << RXCIE0);
  UCSR0B = (1 << RXEN0) | (1 << TXEN0);

  /* Set frame format: 8data, 1stop bit 8N1
   * 0 << USBS0 = 1 stop bit
   * 1 << USBS0 = 2 stop bits (p.193)
   *
   * 3 << UCSZ00 = 8 data bits (p.193)
   * */
  //UCSR0C = (0 << USBS0) | (3 << UCSZ00);
  UCSR0C = (3 << UCSZ00);
}

void UARTInitDynamic(int baud, char flags)
{
}

/**
 * Sends one char through USART0
 * \param char - the char to be send
 */
void UARTPutChar(char data)
{
  /* Wait for empty transmit buffer */
  while (!(UCSR0A & (1 << UDRE0)));
  /* Put data into buffer, sends the data */
  UDR0 = data;
}


unsigned char UARTGetChar(void)
{
  // while (!(UCSRA & (1<<RXC)));  // warten bis Zeichen verfuegbar
  //return UDR;                   // Zeichen aus UDR an Aufrufer zurueckgeben
  while (!(UCSR0A & (1<<RXC0)));  // warten bis Zeichen verfuegbar
    return UDR0;  
}

void UARTWrite(char* msg)
{
  while(*msg != '\0')
  {
     UARTPutChar (*msg++);
  }
}

unsigned char AsciiToHex(unsigned char high,unsigned char low)
{
  unsigned char new;

  // check if lower equal 9 ( ascii 57 )
  if(high <= 57) // high is a number
    high = high -48;
  else // high is a letter
    high = high -87;

  high = high << 4;
  high = high & 0xF0;
 
  // check if lower equal 9 ( ascii 57 )
  if(low <= 57) // high is a number
    low = low -48;
  else // high is a letter
    low = low -87;
  low = low & 0x0F;
 
  new = high | low;
 
  return new;
}

void SendHex(unsigned char hex)
{
  unsigned char high,low;
  // get highnibble
  high = hex & 0xF0;
  high = high >> 4;
 
  // get lownibble
  low = hex & 0x0F;
 
  if(high<=9)
    UARTPutChar(high+48);
  else
    UARTPutChar(high+87);
 
 
  if(low<=9)
    UARTPutChar(low+48);
  else
    UARTPutChar(low+87);

}

